/* Uses Windows-specific functions to send and receive data from the virtual
   serial port of a Pololu Simple Motor Controller.
   NOTE: The Simple Motor Controller's Input Mode must be set to Serial/USB.
   NOTE: You might need to change the 'portName = ' line below. */

#include <stdio.h>
#include <windows.h>

/** Opens a handle to a serial port in Windows using CreateFile.
 * portName: The name of the port.
 * Returns INVALID_HANDLE_VALUE if it fails.  Otherwise returns a handle to the port.
 *   Examples: "COM4", "\\\\.\\USBSER000" */
HANDLE openPort(const char * portName)
{
    HANDLE port;
    BOOL success;
    COMMTIMEOUTS timeouts;

    /* Open the serial port. */
    port = CreateFileA(portName, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if (port == INVALID_HANDLE_VALUE)
    {
        switch(GetLastError())
        {
        case ERROR_ACCESS_DENIED:
            fprintf(stderr, "Error: Access denied.  Try closing all other programs that are using the device.\n");
            break;
        case ERROR_FILE_NOT_FOUND:
            fprintf(stderr, "Error: Serial port not found.  "
                "Make sure that \"%s\" is the right port name.  "
                "Try closing all programs using the device and unplugging the "
                "device, or try rebooting.\n", portName);
            break;
        default:
            fprintf(stderr, "Error: Unable to open serial port.  Error code 0x%x.\n", GetLastError());
            break;
        }
        return INVALID_HANDLE_VALUE;
    }

    /* Set the timeouts. */
    success = GetCommTimeouts(port, &timeouts);
    if (!success)
    {
        fprintf(stderr, "Error: Unable to get comm timeouts.  Error code 0x%x.\n", GetLastError());
        CloseHandle(port);
        return INVALID_HANDLE_VALUE;
    }
    timeouts.ReadIntervalTimeout = 1000;
    timeouts.ReadTotalTimeoutConstant = 1000;
    timeouts.ReadTotalTimeoutMultiplier = 0;
    timeouts.WriteTotalTimeoutConstant = 1000;
    timeouts.WriteTotalTimeoutMultiplier = 0;
    success = SetCommTimeouts(port, &timeouts);
    if (!success)
    {
        fprintf(stderr, "Error: Unable to set comm timeouts.  Error code 0x%x.\n", GetLastError());
        CloseHandle(port);
        return INVALID_HANDLE_VALUE;
    }

    /* Flush out any bytes received from the device earlier. */

    success = FlushFileBuffers(port);
    if (!success)
    {
        fprintf(stderr, "Error: Unable to flush port buffers.  Error code 0x%x.\n", GetLastError());
        CloseHandle(port);
        return INVALID_HANDLE_VALUE;
    }

    return port;
}

#define SERIAL_ERROR -9999

/* Reads a variable from the SMC and returns it as number between 0 and 65535.
   Returns SERIAL_ERROR if there was an error.
   The 'variableId' argument must be one of IDs listed in the
   "Controller Variables" section of the user's guide.
   For variables that are actually signed, additional processing is required
   (see smcGetTargetSpeed for an example). */
int smcGetVariable(HANDLE port, unsigned char variableId)
{
    unsigned char command[] = {0xA1, variableId};
    unsigned char response[2];
    BOOL success;
    DWORD bytesTransferred;

    // Send the command to the device.
    success = WriteFile(port, command, sizeof(command), &bytesTransferred, NULL);
    if (!success)
    {
        fprintf(stderr, "Error: Unable to write Get Variable command to serial port.  Error code 0x%x.\n", GetLastError());
        return SERIAL_ERROR;
    }
    if (sizeof(command) != bytesTransferred)
    {
        fprintf(stderr, "Error: Expected to write %d bytes but only wrote %d.\n", sizeof(command), bytesTransferred);
        return SERIAL_ERROR;
    }

    // Read the response from the device.
    success = ReadFile(port, response, sizeof(response), &bytesTransferred, NULL);
    if (!success)
    {
        fprintf(stderr, "Error: Unable to read Get Variable response from serial port.  Error code 0x%x.\n", GetLastError());
        return SERIAL_ERROR;
    }
    if (sizeof(response) != bytesTransferred)
    {
        fprintf(stderr, "Error: Expected to read %d bytes but only read %d (timeout).\n", sizeof(response), bytesTransferred);
        return SERIAL_ERROR;
    }

    return response[0] + 256*response[1];
}

/* Returns the target speed (-3200 to 3200).
   Returns SERIAL_ERROR if there is an error. */
int smcGetTargetSpeed(HANDLE port)
{
  int val = smcGetVariable(port, 20);
  return val == SERIAL_ERROR ? SERIAL_ERROR : (signed short)val;
}

/* Returns a number where each bit represents a different error, and the
   bit is 1 if the error is currently active.
   See the user's guide for definitions of the different error bits.
   Returns SERIAL_ERROR if there is an error. */
int smcGetErrorStatus(HANDLE port)
{
    return smcGetVariable(port, 0);
}

/* Sends the Exit Safe Start command, which is required to drive the motor.
   Returns 0 if successful, SERIAL_ERROR if there was an error sending. */
int smcExitSafeStart(HANDLE port)
{
    const unsigned char command = 0x83;
    BOOL success;
    DWORD bytesTransferred;

    // Send the command to the device.
    success = WriteFile(port, &command, 1, &bytesTransferred, NULL);
    if (!success)
    {
        fprintf(stderr, "Error: Unable to write Exit Safe Start command to serial port.  Error code 0x%x.\n", GetLastError());
        return SERIAL_ERROR;
    }
    if (1 != bytesTransferred)
    {
        fprintf(stderr, "Error: Expected to write 1 byte but only wrote %d.\n", bytesTransferred);
        return SERIAL_ERROR;
    }
    return 0;
}

/* Sets the SMC's target speed (-3200 to 3200).
   Returns 0 if successful, SERIAL_ERROR if there was an error sending. */
int smcSetTargetSpeed(HANDLE port, int speed)
{
    unsigned char command[3];
    DWORD bytesTransferred;
    BOOL success;

    // Compose the command.
    if (speed < 0)
    {
      command[0] = 0x86; // Motor Reverse
      speed = -speed;
    }
    else
    {
      command[0] = 0x85; // Motor Forward
    }
    command[1] = speed & 0x1F;
    command[2] = speed >> 5 & 0x7F;

    // Send the command to the device.
    success = WriteFile(port, command, sizeof(command), &bytesTransferred, NULL);
    if (!success)
    {
        fprintf(stderr, "Error: Unable to write Set Target command to serial port.  Error code 0x%x.\n", GetLastError());
        return SERIAL_ERROR;
    }
    if (sizeof(command) != bytesTransferred)
    {
        fprintf(stderr, "Error: Expected to write %d bytes but only wrote %d.\n", sizeof(command), bytesTransferred);
        return SERIAL_ERROR;
    }

    return 0;
}

/** This is the first function to run when the program starts. */
int main(int argc, char * argv[])
{
    HANDLE port;
    char * portName;
    int speed, newSpeed;

    /* portName should be the name of the Maestro's Command Port (e.g. "COM4")
     * as shown in your computer's Device Manager.
     * Alternatively you can use \\.\USBSER000 to specify the first virtual COM
     * port that uses the usbser.sys driver.  This will usually be the Maestro's
     * command port. */
    portName = "\\\\.\\USBSER000";  // Each double slash in this source code represents one slash in the actual name.

    /* Open the Simple Motor Controller's serial port. */
    port = openPort(portName);
    if (port == INVALID_HANDLE_VALUE){ return -1; }

    smcExitSafeStart(port);

    printf("Error status: 0x%04x\n", smcGetErrorStatus(port));

    speed = smcGetTargetSpeed(port);
    printf("Current Target Speed is %d.\n", speed);

    newSpeed = (speed <= 0) ? 3200 : -3200;
    printf("Setting Target Speed to %d.\n", newSpeed);
    smcSetTargetSpeed(port, newSpeed);

    CloseHandle(port);
    return 0;
}


